@@ -14,6 +14,7 @@
  *******************************************************************************/
 package org.eclipse.team.internal.ui.history;
 
+import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Date;
 import java.util.List;
@@ -74,9 +75,6 @@ import org.eclipse.ui.part.PluginTransfer;
 import org.eclipse.ui.part.ResourceTransfer;
 import org.eclipse.ui.part.ShowInContext;
 
-import com.ibm.icu.text.SimpleDateFormat;
-
-
 public class GenericHistoryView extends PageBookView implements IHistoryView, IPropertyChangeListener, IShowInTarget {
 	private static final String HISTORY_VIEW_GROUP = "org.eclipse.team.ui.historyView"; //$NON-NLS-1$
 	private static final String NAVIGATION_GROUP = "org.eclipse.team.ui.navigation"; //$NON-NLS-1$
@@ -218,7 +216,7 @@ public class GenericHistoryView extends PageBookView implements IHistoryView, IP
 		}
 	}
 
-	abstract class MenuCreator implements IMenuCreator {
+	abstract static class MenuCreator implements IMenuCreator {
 		private MenuManager menuManager;
 
 		@Override
@@ -382,7 +380,7 @@ public class GenericHistoryView extends PageBookView implements IHistoryView, IP
 			editorActivated((IEditorPart) part);
 	}
 
-	private ISelectionListener selectionListener = new ISelectionListener() {
+	private final ISelectionListener selectionListener = new ISelectionListener() {
 		private boolean isUpdatingSelection = false;
 
 		@Override
